# -*- coding: utf-8 -*-
from datetime import datetime

from django.contrib.syndication.views import Feed
from django.db.models import Q
from django.shortcuts import render
from django.utils import timezone
from django.utils.feedgenerator import Rss201rev2Feed
from django.http import HttpResponse
from django.views.generic.detail import DetailView
from django.utils.html import strip_tags

from jmb.core.views.odt import PdfTemplateView
from publieditor.content.models import Content


class NewsContainer(object):
    """
    Used as interface for classes where get_news method is mandatory.
    """
    
    def get_news(self, universe):
        """ return: an iterable of News objects.
        """
        max_num = universe.max_num or 10
        ordering = universe.get_content_ordering()
        constraining = universe.get_content_constraining()
        streams = universe.streams.all()
        layouts = universe.layouts.all()
        kinds = universe.kinds.all()
        brands = universe.brands.all()
        exclude_streams = universe.exclude_streams
        exclude_layouts = universe.exclude_layouts
        exclude_kinds = universe.exclude_kinds
        request = self.request

        news = Content.objects.select_related(
            'translations',
            'media_photos',
            'car_details',
            'media_videos'
        ).exclude(
            translations__body__isnull=True,
        ).filter(
            status='on', date_content__lte=timezone.now(),
            translations__language=universe.language,
            translations__isnull=False
        )

        if constraining:
            news = news.filter(eval(constraining, globals(), locals()))

        if streams:
            obj_q = Q()
            if exclude_streams:
                for stream in streams:
                    obj_q &= ~Q(stream=stream)
            else:
                for stream in streams:
                    obj_q |= Q(stream=stream)

            news = news.filter(obj_q)

        if layouts:
            obj_q = Q()
            if exclude_layouts:
                for layout in layouts:
                    obj_q &= ~Q(layout=layout)
            else:
                for layout in layouts:
                    obj_q |= Q(layout=layout)
                    
            news = news.filter(obj_q)

        if kinds:
            obj_q = Q()
            if exclude_kinds:
                for kind in kinds:
                    obj_q &= ~Q(kind=kind)
            else:
                for kind in kinds:
                    obj_q |= Q(kind=kind)
                    
            news = news.filter(obj_q)

        if brands:
            obj_q = Q()
            for brand in brands:
                obj_q |= Q(car_details__brand=brand)
                    
            news = news.filter(obj_q).distinct()

        search_queries = request.GET.getlist('query')
        search_queryset = None
        for search_query in search_queries:
            words_queryset = None
            for keyword in search_query.split(" "):
                if isinstance(keyword, (str, unicode)) and keyword.strip() != "":
                    keyword_search_q = (
                        Q(translations__title__icontains=keyword) |
                        Q(translations__body__icontains=keyword)
                    )
                    words_queryset = words_queryset & keyword_search_q if words_queryset else keyword_search_q
            search_queryset = search_queryset | words_queryset if search_queryset else words_queryset
        if search_queryset is not None:
            news = news.filter(search_queryset)

        if ordering:
            news = news.order_by(*ordering)
        else:
            news = news.order_by('-date_content', '-id')

        count = news.count()
        return news[:max_num], count


class GenericFeederView(DetailView, NewsContainer):

    def get_context_data(self, **kwargs):
        self.object = self.get_object()
        ctx = super(GenericFeederView, self).get_context_data(**kwargs)
        self.universe = self.get_object()
        ctx['title'] = self.universe.title or ''
        ctx['img_format'] = self.universe.img_format
        ctx['news'], ctx['news_count'] = self.get_news(self.universe)
        return ctx

    def get_object(self):
        self.universe = self.kwargs['universe']
        self.path = self.kwargs['path']
        return self.universe


class GenericPDFFeed(PdfTemplateView, NewsContainer):
    forceOoCall = True
    attachment = False

    def debug_template(self, template, context):
        template.save_as('/tmp/output.odt', context=context)

    def get_context_data(self, **kwargs):
        universe = self.kwargs['universe']
        news, count = self.get_news(universe)
        
        ## date_issue requires 1th, 2nd, 3rd end the like...
        def suffix(d):
            return 'th' if 11 <= d <= 13 else {1: 'st', 2: 'nd', 3: 'rd'}.get(d % 10, 'th')

        issue_date = news[0].date_content_ordinal()
        ## add a counter
        for j, news_item in enumerate(news):
            news_item.j = j+1

        self.template_name = universe.template_name or self.template_name
        return super(GenericPDFFeed, self).get_context_data(
            pagesize="A4",
            news=news, 
            issue_date=issue_date,
            **kwargs
        )


class GenericHtmlMixin(object):
    template_name = 'universe/feeds/generic_html.html'
    title = 'AutolinkNews'

    def render_to_response(self, context, **response_kwargs):
        news, news_count = self.get_news(self.universe)
        return render(
            self.request, self.template_name, 
            dict(
                universe=self.universe,
                title=self.title,
                news=news
            ),
            content_type="text/html"
        )


class StreamResponseMixin(object):
    """
    A mixin that can be used to render a stream response.
    """
    template_name = "universe/feeds/generic_feed.xml"
    title = 'Autolink News'

    def render_to_response(self, context, **response_kwargs):
        """
        Returns a response with a template rendered with the given context.
        """
        news, news_count = self.get_news(self.universe)
        now = datetime.now().strftime("%a, %d %b %Y %H:%M:%S %Z") + '%+03d%02d' % (0, 0)
        return render(
            self.request, self.template_name,
            dict(
                title=self.title,
                news=news,
                today_rfc=now,
                universe=self.universe
            ),
            content_type="application/xml"
        )


class RssFlipboardResponseMixin(StreamResponseMixin):
    """

    """
    template_name = "universe/feeds/generic_rss_fipboard.xml"


class RssProResponseMixin(StreamResponseMixin):
    """

    """
    template_name = "universe/feeds/generic_rss_pro.xml"


class UniverseRssFeed(Rss201rev2Feed, NewsContainer):
    """Uses RSS2.0 feeds with elements from Atom and MRSS namespace
    also.  This allows to add multiple images to one item, bypassing
    the enclosure element limit of having only one per element.

    """

    def rss_attributes(self):
        attrs = super(UniverseRssFeed, self).rss_attributes()
        attrs['xmlns:media'] = 'http://search.yahoo.com/mrss/'
        attrs['xmlns:atom'] = 'http://www.w3.org/2005/Atom'
        return attrs

    def add_root_elements(self, handler):
        """Callback to add elements inside the root feed element
        (feed/channel). handler is an XMLGenerator from Python’s
        built-in SAX library; you’ll call methods on it to add to the
        XML document in process.

        """

        super(UniverseRssFeed, self).add_root_elements(handler)
        handler.startElement('image', {})
        handler.addQuickElement(u'url', u'http://www.autolinknews.com/it/_img/atlk_rss.png')
        handler.addQuickElement(u'title', self.feed['title'])
        handler.addQuickElement(u'link', self.feed['link'])
        handler.endElement('image')

    def add_item_elements(self, handler, item):
        import mimetypes

        super(UniverseRssFeed, self).add_item_elements(handler, item)

        # add media contents
        universe = item["universe"]
        for pic in item['photos']:
            url_pic = pic['orig']
            if universe.img_format == "orig16_9" and pic['orig16_9']:
                url_pic = pic['orig16_9']
            photo_mime_type = mimetypes.guess_type(pic['orig'])[0]
            handler.startElement('media:content', dict(
                type=(photo_mime_type or 'image/jpg'),
                url=url_pic
            ))
            handler.endElement('media:content')

        for vid in item['videos']:
            video_mime_type = mimetypes.guess_type(vid['orig'])[0]
            handler.startElement('media:content', dict(
                type=(video_mime_type or 'video/mp4'),
                url=vid['orig']
            ))
            handler.endElement('media:content')


class GenericRSSFeed(Feed, NewsContainer):
    link = "http://www.autolinknews.com"
    description = "Il primo quotidiano sull'industria dell'auto, fondato da Pietro Rizzo"
    
    copyright = "©2001-2010 www.autolinknews.com"

    feed_type = UniverseRssFeed
        
    def get_object(self, request, *args, **kwargs):
        self.universe = kwargs['universe']
        self.request = request
        return kwargs['universe']

    def title(self, universe):
        title = universe.title or "AutolinkNews"
        return title
        
    def items(self, universe):

        news, news_count = self.get_news(universe)
        self.rich_text = universe.rich_text

        ret = []
        for n in news:
            transall = n.translations.all()
            if transall:
                ret.append(transall[0])
            if len(ret) > (universe.max_num or 10):
                break

        return ret

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        if self.rich_text:
            return item.stripped_body
        return strip_tags(item.stripped_body)

    def item_link(self, item):
        return item.get_detail_url()

    def item_pubdate(self, item):
        return item.content.date_content

    def feed_extra_kwargs(self, universe):
        return dict(image=dict(url=u'http://image/url.png', title=self.title, link=self.link))

    def item_extra_kwargs(self, item):
        extra_dict = {}
        extra_dict.update(super(GenericRSSFeed, self).item_extra_kwargs(item))
        extra_dict.update(dict(
            universe=self.universe,
            photos=item.content.get_photo_url_list(),
            videos=item.content.get_video_url_list(),
        ))

        return extra_dict
    
    # def item_enclosure_url(self, item):
    #     photo_url = item.content.get_photo_url()
    #     if photo_url:
    #         return photo_url
    #
    # def item_enclosure_mime_type(self, item):
    #     import mimetypes
    #
    #     photo_url = item.content.get_photo_url()
    #     if photo_url:
    #         photo_mime_type = mimetypes.guess_type(photo_url)[0]
    #
    #         if photo_mime_type:
    #             return photo_mime_type


class PListResponseMixin(object):
    """
    A mixin that can be used to render a JSON response.
    """
    template_name = 'universe/feeds/generic_plist.xml'

    def get_articles(self, news):
        articles = []
        for curr_news in news:
            transall = curr_news.translations.all()
            brands = curr_news.get_brand_names_list()
            if transall:
                translation = transall[0]
                pub_date = curr_news.date_content.date()
                pub_datetime = curr_news.date_content
                kind = curr_news.get_kind(self.universe)
                summary = translation.stripped_body if self.universe.rich_text else strip_tags(translation.stripped_body)

                photo = thumb = photo_width = photo_height = ''
                if curr_news.get_first_photo_url_dict():
                    photo = curr_news.get_first_photo_url_dict()['orig'] + '?' + pub_datetime.strftime('%Y%m%d%H%M%S')
                    thumb = curr_news.get_first_photo_url_dict()['thumb'] + pub_datetime.strftime('%Y%m%d%H%M%S')
                    photo_width = curr_news.get_first_photo().width or ''
                    photo_height = curr_news.get_first_photo().height or ''

                article = dict(
                    ID=str(curr_news.id),
                    articleId=unicode(curr_news.code),
                    page=unicode(curr_news.page) if curr_news.page else '',
                    issue=unicode(getattr(curr_news.issue, 'number', '')),
                    pub_date=unicode(pub_date),
                    date='',
                    time=pub_date.strftime('%d/%m/%Y'),
                    title=translation.title,
                    summary=summary,
                    kind=kind.name if kind else '',
                    autolink_kind=kind.name if kind else '',
                    autolink_kind_id=kind.id if kind else '',
                    sector=curr_news.sector.name if curr_news.sector else '',
                    brand1=brands[0] if len(brands) else '',
                    layout=curr_news.layout_id,
                    photo=photo,
                    photo_width=photo_width,
                    photo_height=photo_height,
                    thumb=thumb,
                    udid='',
                )

                if curr_news.get_media_photos():
                    article['files'] = []
                    for photo in curr_news.get_media_photos():
                        if self.universe and self.universe.img_format == "orig16_9":
                            if photo.has16_9:
                                photo_url = photo.get_photo_url_dict()['orig16_9']
                            else:
                                photo_url = photo.get_photo_url_dict()['orig']
                        else:
                            photo_url = photo.get_photo_url_dict()['orig']
                        ext = photo.get_photo_type_dict()["orig"]
                        if photo.get_photo_type_dict()["orig"] == "jpeg":
                            ext = "jpg"
                        if photo_url:
                            article['files'].append(dict(
                                dida=photo.title or '',
                                stream_name=photo.title or '',
                                stream_full_path=photo_url,
                                stream_ext=ext or '',
                                stream_width=photo.width or '',
                                stream_height=photo.height or '',
                                # poster_name=photo.title,
                                # poster_full_path=photo_url,
                                # poster_ext=u'*** POSTER EXT'
                            ))

                if curr_news.get_media_videos():
                    if 'files' not in article:
                        article['files'] = []
                    for video in curr_news.get_media_videos():
                        video_url = video.get_video_url_dict()['orig']
                        ext = video.get_video_type_dict()["orig"]
                        if video_url:
                            article['files'].append(dict(
                                dida=video.title or '',
                                stream_name=video.title or '',
                                stream_full_path=video_url,
                                stream_ext=ext or '',
                                stream_width=video.width or '',
                                stream_height=video.height or '',
                            ))

                articles.append(article)
        return articles

    def render_to_response(self, context, **response_kwargs):
        """
        Returns a plist response.
        """
        import plistlib
        news, news_count = self.get_news(self.universe)
        doc = dict(
            found_rows=str(news_count),
            udid=''
        )
        doc['articles'] = self.get_articles(news)
        response = HttpResponse(content_type='application/xml')
        plistlib.writePlist(doc, response)
        return response


