# coding: utf-8
"""Universe Models"""
import hashlib

from django.conf import settings
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.text import slugify
from django.utils.translation import ugettext_lazy as _

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, SEOModel
from jmb.i18n.models import TranslationModel, I18NModel, Language
from jmb.core.db.fields import HTMLField, HTMLFieldTinyMCE

from publieditor.content.models import Stream, Layout, Kind, Brand

from tokenize import generate_tokens, untokenize
import token

import re
from cStringIO import StringIO


class Universe(DateModel, UserModel, OrderedModel):
    # __rc__split__filter =  re.compile(r'^\s*(\w+)\s*=\s*(\w+)\s*$')

    def __compile_constraint__(self):
        if self.constrain and self._compiled_constraint is None:
            codestream = StringIO('(' + re.sub(r'[\s\n\r]+', '', self.constrain) + ')')

            outtok = []
            for toknum, tokval, _, _, _ in generate_tokens(codestream.readline):
                if toknum == token.OP and tokval == '(':
                    outtok.append((token.NAME, 'Q'))
                outtok.append((toknum, tokval,))

            self._compiled_constraint = compile(untokenize(outtok), '<string>', 'eval')

        return self._compiled_constraint

    def __init__(self, *args, **kwargs):
        super(Universe, self).__init__(*args, **kwargs)
        self._compiled_constraint = None
        
    name = models.CharField(
        max_length=100, unique=True,
        verbose_name=_('name'))
    title = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_('title'))
    status = models.CharField(
        choices=settings.UNIVERSE_UNIVERSE_STATUS_CODES,
        max_length=3, default='on', verbose_name=_('status')
    )
    language = models.ForeignKey(
        Language,
        related_name='universes',
        verbose_name=_('language'),
    )
    check = models.BooleanField(
        verbose_name=_('check'),
    )
    output = models.CharField(
        choices=settings.UNIVERSE_UNIVERSE_OUTPUT_CHOICES,
        max_length=30,
        verbose_name=_('output'),
    )
    doc_type = models.CharField(
        choices=settings.UNIVERSE_UNIVERSE_DOC_TYPE_CHOICES,
        max_length=30,
        blank=True, null=True,
        verbose_name=_('doc type'),
    )
    mime_type = models.CharField(
        choices=settings.UNIVERSE_UNIVERSE_MIME_TYPE_CHOICES,
        max_length=30,
        blank=True, null=True,
        verbose_name=_('mime type'),
    )
    domain = models.URLField(
        default='http://www.autolinknews.com',
        verbose_name=_('domain'),
    )
    username = models.CharField(
        max_length=30,
        blank=True, null=True,
        verbose_name=_('username'),
    )
    password = models.CharField(
        max_length=30,
        blank=True, null=True,
        verbose_name=_('password'),
    )
    secret_phrase = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_('secret phrase'),
    )
    path = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_('path'),
    )
    folder_name = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_('folder name'),
    )
    file_name = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_('file name'),
    )
    max_num = models.PositiveIntegerField(
        blank=True, null=True,
        verbose_name=_('max num'),
    )
    rich_text = models.BooleanField(
        default=False,
        verbose_name=_('rich text'),
    )
    content_ordering = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_('content ordering'),
    )
    streams = models.ManyToManyField(
        Stream,
        related_name='universes',
        blank=True, null=True,
        verbose_name=_('streams'),
    )
    exclude_streams = models.BooleanField(
        verbose_name=_('exclude streams'),
    )
    layouts = models.ManyToManyField(
        Layout,
        related_name='universes',
        blank=True, null=True,
        verbose_name=_('layouts'),
    )
    exclude_layouts = models.BooleanField(
        verbose_name=_('exclude layouts'),
    )
    kinds = models.ManyToManyField(
        Kind,
        related_name='universes',
        blank=True, null=True,
        verbose_name=_('kinds'),
    )
    exclude_kinds = models.BooleanField(
        verbose_name=_('exclude kinds'),
    )
    brands = models.ManyToManyField(
        Brand,
        related_name='universes',
        blank=True, null=True,
        verbose_name=_('brands'),
    )
    template_name = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_('template name'),
    )
    constrain = models.TextField(
        blank=True, null=True,
        verbose_name=_('constrain'),
    )
    features = models.TextField(
        blank=True, null=True,
        verbose_name=_('features'),
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_('note'),
    )
    img_format = models.CharField(
        max_length=10,
        choices=settings.UNIVERSE_UNIVERSE_IMAGE_FORMAT_CHOICES,
        default=settings.UNIVERSE_UNIVERSE_IMAGE_FORMAT_4_3_KEY,
        verbose_name=_('image format'),
    )

    class Meta:
        verbose_name = _('Universe')
        verbose_name_plural = _('Universes')
        ordering = ('ordering', 'name', 'title')

    def __unicode__(self):
        return u"%s - %s" % (self.get_status_display(), self.name)

    def get_url(self):
        # set profile
        profile = ""
        if self.username and self.password:
            profile = "%s:%s@" % (self.username, self.password)

        # set protocol and domain
        protocol, domain = self.domain.split('://')

        # set secret phrase
        secret = ''
        if self.get_secret():
            secret = "/%s" % self.get_secret()
        return "%s://%s%s/feed/%s%s" % (protocol, profile, domain, self.path if self.path else '', secret)
    get_url.short_description = _('url')

    def get_secret(self):
        secret = None
        if self.secret_phrase:
            secret = "%s" % hashlib.sha1(slugify(self.secret_phrase)).hexdigest()
        return secret

    def check_secret(self, hash_secret):
        is_valid = False
        secret = self.get_secret()
        if secret is not None:
            if secret == hash_secret:
                is_valid = True
        else:
            is_valid = True
        return is_valid

    def get_path(self):
        # set profile
        profile = ""
        if self.username and self.password:
            profile = "%s:%s@" % (self.username, self.password)
            # return "%s%s" % (profile, self.path if self.path else '')

        # set secret phrase
        secret = ""
        if self.secret_phrase:
            secret = "/%s" % hashlib.sha1(slugify(self.secret_phrase)).hexdigest()
            return "%s%s" % (self.path if self.path else '', secret)

        return self.path or ''
    get_path.short_description = _('path')

    def get_content_ordering(self):
        ret = ['-date_content']
        if self.content_ordering:
            ret = [x.strip() for x in self.content_ordering.split(',')]
        return ret

    def get_content_constraining(self):
        return self.__compile_constraint__()

    def get_simple_url(self):
        pass